/*!

Copyright 2022 Mia srl

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

	http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.

*/
"use strict";
var __defProp = Object.defineProperty;
var __defNormalProp = (obj, key, value) => key in obj ? __defProp(obj, key, { enumerable: true, configurable: true, writable: true, value }) : obj[key] = value;
var __publicField = (obj, key, value) => {
  __defNormalProp(obj, typeof key !== "symbol" ? key + "" : key, value);
  return value;
};
const COMPOSER_BODY_CLASS = "composer-body";
const isSameApplication = (current, next) => {
  if (current === void 0 && next === void 0) {
    return false;
  }
  if (current === void 0 || next === void 0) {
    return false;
  }
  return JSON.stringify(current) === JSON.stringify(next);
};
class MicrofrontendLoader extends HTMLElement {
  constructor() {
    super(...arguments);
    __publicField(this, "container", this);
    __publicField(this, "state", { application: void 0, loader: void 0 });
    __publicField(this, "initialResolve");
    __publicField(this, "queue", new Promise((resolve) => {
      this.initialResolve = resolve;
    }));
    __publicField(this, "microlcApi");
  }
  enqueue(fn) {
    this.queue = this.queue.then(() => {
      const result = fn();
      return result !== null && typeof result === "object" && "then" in result ? result : Promise.resolve(result);
    }).then(() => {
    });
  }
  set application(application) {
    if (!isSameApplication(this.state.application, application)) {
      this.enqueue(() => this.update("application", { application }));
    }
  }
  get application() {
    return this.state.application;
  }
  connectedCallback() {
    var _a, _b;
    let renderRoot = this;
    const disableShadowDom = this.getAttribute("disable-shadow-dom") !== null;
    const disableStyling = this.getAttribute("disable-styling") !== null;
    if (this.state.loader === void 0) {
      this.state.loader = (_a = this.microlcApi) == null ? void 0 : _a.createLoader();
    }
    const id = "_".concat(window.crypto.randomUUID());
    const container = Object.assign(
      this.ownerDocument.createElement("div"),
      { id }
    );
    const style = Object.assign(
      this.ownerDocument.createElement("style"),
      {
        textContent: "\n          div#".concat(id, " {\n            width: 100%;\n            height: 100%;\n            overflow: hidden;\n          }\n\n          div#").concat(id, " > :first-child {\n            width: inherit;\n            height: inherit;\n            overflow: hidden;\n          }\n\n          div#").concat(id, " > :first-child > div.").concat(COMPOSER_BODY_CLASS, " {\n            width: inherit;\n            height: inherit;\n            overflow: hidden;\n          }\n        ")
      }
    );
    if (!disableShadowDom) {
      renderRoot = this.attachShadow({ mode: "open" });
    }
    !disableStyling && renderRoot.appendChild(style);
    this.container = renderRoot.appendChild(container);
    (_b = this.initialResolve) == null ? void 0 : _b.call(this);
  }
  async update(name, next) {
    var _a, _b;
    if (name === "application") {
      await ((_a = this.state.loader) == null ? void 0 : _a.unload());
      if (next.application !== void 0) {
        await ((_b = this.state.loader) == null ? void 0 : _b.load(next.application, this.container));
      }
      this.state.application = next.application;
    }
  }
  disconnectedCallback() {
    const { state: { loader } } = this;
    this.enqueue(() => loader == null ? void 0 : loader.unload());
  }
}
customElements.define("microfrontend-loader", MicrofrontendLoader);
