/*!

Copyright 2022 Mia srl

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

	http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.

*/
"use strict";
var __create = Object.create;
var __defProp = Object.defineProperty;
var __getOwnPropDesc = Object.getOwnPropertyDescriptor;
var __getOwnPropNames = Object.getOwnPropertyNames;
var __getProtoOf = Object.getPrototypeOf;
var __hasOwnProp = Object.prototype.hasOwnProperty;
var __export = (target, all) => {
  for (var name in all)
    __defProp(target, name, { get: all[name], enumerable: true });
};
var __copyProps = (to, from, except, desc) => {
  if (from && typeof from === "object" || typeof from === "function") {
    for (let key of __getOwnPropNames(from))
      if (!__hasOwnProp.call(to, key) && key !== except)
        __defProp(to, key, { get: () => from[key], enumerable: !(desc = __getOwnPropDesc(from, key)) || desc.enumerable });
  }
  return to;
};
var __toESM = (mod, isNodeMode, target) => (target = mod != null ? __create(__getProtoOf(mod)) : {}, __copyProps(
  // If the importer is in node compatibility mode or this is not an ESM
  // file that has been converted to a CommonJS file using a Babel-
  // compatible transform (i.e. "__esModule" has not been set), then set
  // "default" to the CommonJS "module.exports" for node compatibility.
  isNodeMode || !mod || !mod.__esModule ? __defProp(target, "default", { value: mod, enumerable: true }) : target,
  mod
));
var __toCommonJS = (mod) => __copyProps(__defProp({}, "__esModule", { value: true }), mod);
var json_exports = {};
__export(json_exports, {
  invalidJsonCatcher: () => invalidJsonCatcher,
  jsonFetcher: () => jsonFetcher,
  jsonToObject: () => jsonToObject,
  jsonToObjectCatcher: () => jsonToObjectCatcher
});
module.exports = __toCommonJS(json_exports);
var import_logger = __toESM(require("../logger/index.js"), 1);
var import_array = require("./array.js");
const jsonAcceptedTypes = [
  "application/json",
  "text/x-json"
];
const yamlAcceptedTypes = [
  "application/yaml",
  "application/x-yaml",
  "text/yaml"
];
const acceptedTypes = [
  ...jsonAcceptedTypes,
  ...yamlAcceptedTypes
];
function isSchemaOptions(input) {
  return Object.prototype.hasOwnProperty.call(input, "id") && Object.prototype.hasOwnProperty.call(input, "parts");
}
async function jsonFetcher(url, init) {
  return globalThis.fetch(
    new URL(url, globalThis.document.baseURI),
    {
      ...init,
      headers: {
        ...init == null ? void 0 : init.headers,
        Accept: acceptedTypes.join(", ")
      }
    }
  ).then((res) => {
    var _a;
    const contentType = (_a = res.headers.get("Content-Type")) != null ? _a : "";
    const isJson = jsonAcceptedTypes.reduce(
      (accepted, str) => contentType.includes(str) || accepted,
      false
    );
    const isYaml = yamlAcceptedTypes.reduce(
      (accepted, str) => contentType.includes(str) || accepted,
      false
    );
    if (res.ok && isJson) {
      return res.json();
    }
    if (res.ok && isYaml) {
      return Promise.all([import("js-yaml"), res.text()]).then(([{ default: yaml }, data]) => {
        return yaml.load(data, { json: true, schema: yaml.JSON_SCHEMA });
      });
    }
    return Promise.reject(new TypeError("20"));
  });
}
async function jsonToObject(input, schema) {
  if (true) {
    if (schema) {
      return Promise.all([
        import("ajv"),
        import("ajv-formats")
      ]).then(([
        { default: Ajv },
        { default: addFormats }
      ]) => {
        try {
          let schemas = (0, import_array.toArray)(schema);
          let id = schema.$id;
          if (isSchemaOptions(schema)) {
            schemas = schema.parts;
            id = schema.id;
          }
          const ajv = new Ajv({ schemas });
          addFormats(ajv);
          const validate = ajv.getSchema(id);
          validate(input);
          const { errors: cause } = validate;
          if (cause) {
            return Promise.reject(new TypeError("21", { cause }));
          }
        } catch (err) {
          return Promise.reject(new TypeError("22", { cause: err.message }));
        }
        return Promise.resolve(input);
      });
    }
  }
  return Promise.resolve(input);
}
function invalidJsonCatcher(err, data, file) {
  if (err instanceof TypeError) {
    if (err.message === "20") {
      import_logger.default.error(err.message, file != null ? file : '"unknown"');
    } else {
      import_logger.default.error("1", err.message);
    }
  }
  return data;
}
function jsonToObjectCatcher(err, data, file) {
  if (err instanceof TypeError) {
    const handledCodes = ["21", "22"];
    if (handledCodes.includes(err.message)) {
      import_logger.default.error(err.message, file != null ? file : '"unknown"', err.cause ? JSON.stringify(err.cause) : "null");
    } else {
      import_logger.default.error(err.message);
    }
  }
  return data;
}
