/*!

Copyright 2022 Mia srl

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

	http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.

*/
"use strict";
var __defProp = Object.defineProperty;
var __getOwnPropDesc = Object.getOwnPropertyDescriptor;
var __getOwnPropNames = Object.getOwnPropertyNames;
var __hasOwnProp = Object.prototype.hasOwnProperty;
var __export = (target, all) => {
  for (var name in all)
    __defProp(target, name, { get: all[name], enumerable: true });
};
var __copyProps = (to, from, except, desc) => {
  if (from && typeof from === "object" || typeof from === "function") {
    for (let key of __getOwnPropNames(from))
      if (!__hasOwnProp.call(to, key) && key !== except)
        __defProp(to, key, { get: () => from[key], enumerable: !(desc = __getOwnPropDesc(from, key)) || desc.enumerable });
  }
  return to;
};
var __toCommonJS = (mod) => __copyProps(__defProp({}, "__esModule", { value: true }), mod);
var extensions_exports = {};
__export(extensions_exports, {
  initBaseExtensions: () => initBaseExtensions,
  updateCSS: () => updateCSS
});
module.exports = __toCommonJS(extensions_exports);
var import_dom_manipulation = require("../../dom-manipulation/index.js");
var import_json = require("../../utils/json.js");
var import_router = require("./router.js");
var import_update = require("./update.js");
function updateCSS(css) {
  this._styleElements.forEach((style) => {
    style.remove();
  });
  if (this.shadowRoot && "adoptedStyleSheets" in this.ownerDocument) {
    const stylesheets = (0, import_dom_manipulation.createCSSStyleSheets)(css);
    this.shadowRoot.adoptedStyleSheets = [...stylesheets];
  } else {
    const styleTags = Array(2).fill(0).map(
      () => this.ownerDocument.createElement("style")
    );
    this._styleElements = (0, import_dom_manipulation.injectStyleToElements)(css, styleTags, this.shadowRoot !== null);
    this._styleElements.forEach((el) => {
      this.shadowRoot ? this.shadowRoot.insertBefore(el, this.shadowRoot.firstChild) : this.ownerDocument.head.appendChild(el);
    });
  }
}
function initLanguageExtension() {
  let currentLanguage = window.navigator.language;
  return {
    getFallbackLanguage: () => {
      return this._fallbackLanguage;
    },
    getLanguage() {
      return currentLanguage;
    },
    setLanguage: (nextLang) => {
      currentLanguage = nextLang;
      setTimeout(() => {
        this._prepareForUpdate();
        this.update().then((done) => {
          if (done) {
            this.matchCache.invalidateCache();
            this._reroute().catch(import_router.rerouteErrorHandler);
            this._completeUpdate();
          }
        }).catch((err) => (0, import_update.handleUpdateError)(import_router.currentApplication$, err));
      });
    }
  };
}
function initBaseExtensions() {
  return {
    css: {
      setStyle: (css) => {
        updateCSS.call(this, css);
      }
    },
    head: {
      setIcon: ({ href, sizes, type }) => {
        var _a;
        const icon = (_a = this.ownerDocument.head.querySelector("link[rel=icon]")) != null ? _a : this.ownerDocument.createElement("link");
        Object.assign(icon, {
          href: href != null ? href : icon.href,
          sizes: sizes != null ? sizes : icon.sizes,
          type: type != null ? type : icon.type
        });
      },
      setTitle: (nextTitle) => {
        var _a;
        const title = (_a = this.ownerDocument.head.querySelector("title")) != null ? _a : this.ownerDocument.createElement("title");
        title.textContent = nextTitle;
        !title.isConnected && this.ownerDocument.head.appendChild(title);
      }
    },
    httpClient: (input, init) => window.fetch(input, init),
    json: {
      fetcher: import_json.jsonFetcher,
      async validator(json, schema, {
        defaultValue = json,
        file
      } = {}) {
        return (0, import_json.jsonToObject)(json, schema).catch(
          (err) => (0, import_json.jsonToObjectCatcher)(err, defaultValue, file)
        );
      }
    },
    language: initLanguageExtension.call(this)
  };
}
