/*!

Copyright 2022 Mia srl

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

	http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.

*/
"use strict";
var __create = Object.create;
var __defProp = Object.defineProperty;
var __getOwnPropDesc = Object.getOwnPropertyDescriptor;
var __getOwnPropNames = Object.getOwnPropertyNames;
var __getProtoOf = Object.getPrototypeOf;
var __hasOwnProp = Object.prototype.hasOwnProperty;
var __defNormalProp = (obj, key, value) => key in obj ? __defProp(obj, key, { enumerable: true, configurable: true, writable: true, value }) : obj[key] = value;
var __export = (target, all) => {
  for (var name in all)
    __defProp(target, name, { get: all[name], enumerable: true });
};
var __copyProps = (to, from, except, desc) => {
  if (from && typeof from === "object" || typeof from === "function") {
    for (let key of __getOwnPropNames(from))
      if (!__hasOwnProp.call(to, key) && key !== except)
        __defProp(to, key, { get: () => from[key], enumerable: !(desc = __getOwnPropDesc(from, key)) || desc.enumerable });
  }
  return to;
};
var __toESM = (mod, isNodeMode, target) => (target = mod != null ? __create(__getProtoOf(mod)) : {}, __copyProps(
  // If the importer is in node compatibility mode or this is not an ESM
  // file that has been converted to a CommonJS file using a Babel-
  // compatible transform (i.e. "__esModule" has not been set), then set
  // "default" to the CommonJS "module.exports" for node compatibility.
  isNodeMode || !mod || !mod.__esModule ? __defProp(target, "default", { value: mod, enumerable: true }) : target,
  mod
));
var __toCommonJS = (mod) => __copyProps(__defProp({}, "__esModule", { value: true }), mod);
var __publicField = (obj, key, value) => {
  __defNormalProp(obj, typeof key !== "symbol" ? key + "" : key, value);
  return value;
};
var mfe_loader_exports = {};
__export(mfe_loader_exports, {
  MFELoader: () => MFELoader,
  loadApp: () => loadApp,
  prepareApplicationConfig: () => prepareApplicationConfig
});
module.exports = __toCommonJS(mfe_loader_exports);
var import_composer = require("@micro-lc/composer");
var import_array = require("../../utils/array.js");
var import_handler = require("./handler.js");
var import_qiankun = require("./qiankun.js");
const buildComposer = (mode, extraProperties) => {
  const defaultComposerApi = {
    context: extraProperties,
    createComposerContext: import_composer.createComposerContext,
    premount: import_composer.premount
  };
  switch (mode) {
    case "iframe":
      return {
        ...defaultComposerApi,
        createComposerContext: (content, opts) => {
          var _a;
          return (0, import_composer.createComposerContext)(content, {
            context: {
              ...opts == null ? void 0 : opts.context,
              onload() {
              }
            },
            extraProperties: /* @__PURE__ */ new Set([...(_a = opts == null ? void 0 : opts.extraProperties) != null ? _a : [], "onload"])
          });
        },
        premount: import_composer.premount
      };
    case "compose":
      return { context: defaultComposerApi.context };
    case "parcel":
    default:
      return defaultComposerApi;
  }
};
const getExtraIFrameAttributes = (app) => {
  var _a, _b;
  const extraIframeAttributes = {};
  if (typeof ((_a = app.attributes) == null ? void 0 : _a.src) === "string") {
    extraIframeAttributes.src = app.attributes.src;
  }
  if (typeof app.src === "string") {
    extraIframeAttributes.src = app.src;
  }
  if (typeof ((_b = app.attributes) == null ? void 0 : _b.srcdoc) === "string") {
    extraIframeAttributes.srcdoc = app.attributes.srcdoc;
  }
  if (typeof app.srcdoc === "string") {
    extraIframeAttributes.srcdoc = app.srcdoc;
  }
  return extraIframeAttributes;
};
const getParcelEntry = ({ entry }) => {
  var _a, _b;
  if (typeof entry === "string") {
    return entry;
  }
  const scripts = (0, import_array.toArray)((_a = entry.scripts) != null ? _a : []);
  const styles = (0, import_array.toArray)((_b = entry.styles) != null ? _b : []);
  if (typeof entry.html === "string" && scripts.length === 0) {
    return entry.html;
  }
  return {
    ...entry,
    scripts,
    styles
  };
};
async function getApplicationSchema() {
  let schema;
  if (true) {
    schema = await import("../../utils/schemas.js").then((schemas) => ({
      id: schemas.pluginSchema.$id,
      parts: schemas
    }));
  }
  return schema;
}
const prepareApplicationConfig = (id, app, composerUri) => {
  const name = "".concat(id, "-").concat(window.crypto.randomUUID());
  const contextMaker = prepareRoutelessApp(name, app, composerUri);
  return {
    makeConfig: (container, microlcApi, sharedProperties) => {
      var _a;
      const context = contextMaker(container, { ...sharedProperties, microlcApi });
      const fetchConfigOnMount = app.integrationMode !== "compose" ? void 0 : (_a = app.options) == null ? void 0 : _a.fetchConfigOnMount;
      return {
        ...context,
        props: {
          ...context.props,
          options: { fetchConfigOnMount },
          schema: getApplicationSchema()
        }
      };
    },
    name
  };
};
const prepareRoutelessApp = (name, app, composerUri) => {
  let injectBase = false;
  let entry;
  let config;
  let properties;
  switch (app.integrationMode) {
    case "compose":
      if (!composerUri) {
        throw new TypeError("Missing composer uri");
      }
      entry = {
        html: '\n          <!DOCTYPE html>\n          <html>\n          <head></head>\n          <body><div class="'.concat(MFELoader.COMPOSER_BODY_CLASS, '" id="').concat(name, '"></div></body>\n          </html>\n        '),
        scripts: [composerUri],
        styles: []
      };
      config = typeof app.config === "string" ? app.config : { ...app.config };
      break;
    case "iframe":
      if (!composerUri) {
        throw new TypeError("Missing composer uri");
      }
      entry = { scripts: [composerUri] };
      config = {
        content: {
          attributes: {
            style: "width: inherit;\n                    height: inherit;\n                    border: none;",
            ...app.attributes,
            ...getExtraIFrameAttributes(app)
          },
          content: {
            content: "Your browser does not support iframes",
            tag: "p"
          },
          tag: "iframe"
        }
      };
      break;
    case "parcel":
    default:
      injectBase = app.injectBase;
      entry = getParcelEntry(app);
      properties = app.properties;
      break;
  }
  return (container, sharedProperties) => ({
    // qiankun typing is
    // not precise here since a shadow root does
    // work as container of a mfe
    container,
    entry,
    name,
    props: {
      composerApi: buildComposer(app.integrationMode, sharedProperties),
      config,
      injectBase,
      ...sharedProperties,
      ...properties
    }
  });
};
const _MFELoader = class _MFELoader {
  constructor(config, getApi, loader) {
    __publicField(this, "getApi");
    __publicField(this, "loader");
    __publicField(this, "composerUri");
    __publicField(this, "sharedProperties");
    __publicField(this, "handlers");
    this.loader = loader;
    this.composerUri = config.settings.composerUri;
    this.sharedProperties = config.shared.properties;
    this.getApi = getApi;
  }
  getMount(shouldMountByLoading = false) {
    const app = this.handlers;
    if (app) {
      return () => app.loadPromise.then(() => app.bootstrapPromise).then(() => app.getStatus() === "NOT_MOUNTED" && !shouldMountByLoading ? app.mount() : app.mountPromise);
    }
  }
  getUnmount() {
    const app = this.handlers;
    if (app) {
      if (app.getStatus() !== "UNMOUNTING") {
        return () => app.loadPromise.then(() => app.bootstrapPromise).then(() => app.mountPromise).then(() => app.getStatus() === "NOT_MOUNTED" ? app.unmountPromise : app.unmount());
      }
      return () => app.unmountPromise;
    }
  }
  getUpdate() {
    let update;
    const app = this.handlers;
    if (app) {
      update = (props) => app.loadPromise.then(() => app.bootstrapPromise).then(() => app.mountPromise).then(() => {
        var _a;
        return (_a = app.update) == null ? void 0 : _a.call(app, props);
      });
    }
    return update;
  }
  async load(app, container) {
    var _a;
    const shouldMountByLoading = this.handlers === void 0;
    if (shouldMountByLoading) {
      const { makeConfig } = prepareApplicationConfig(_MFELoader.ID, app, this.composerUri);
      const loadableApp = makeConfig(container, this.getApi(), this.sharedProperties);
      this.handlers = this.loader.loadMicroApp(loadableApp, { getPublicPath: import_handler.getPublicPath, sandbox: { speedy: false } });
    }
    await ((_a = this.getMount(shouldMountByLoading)) == null ? void 0 : _a());
    return this.getUpdate();
  }
  async unload() {
    var _a;
    return (_a = this.getUnmount()) == null ? void 0 : _a();
  }
};
__publicField(_MFELoader, "ID", "microfrontend");
__publicField(_MFELoader, "COMPOSER_BODY_CLASS", "composer-body");
let MFELoader = _MFELoader;
const loadApp = (name, app, container, lifeCycles, {
  composerUri,
  ...configuration
}, loader = (0, import_qiankun.createQiankunInstance)()) => {
  var _a;
  const makeConfig = prepareRoutelessApp(name, app, composerUri);
  const loadableApp = makeConfig(container, (_a = app.properties) != null ? _a : {});
  return loader.loadMicroApp(loadableApp, configuration, lifeCycles);
};
