/*!

Copyright 2022 Mia srl

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

	http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.

*/
import composerUri from "@micro-lc/composer/composer.production?url";
import page401Url from "./assets/401.html?url";
import page404Url from "./assets/404.html?url";
import page500Url from "./assets/500.html?url";
import { computeAbsoluteRoute } from "./web-component/lib/index.js";
const MICRO_LC_MOUNT_POINT = "__MICRO_LC_MOUNT_POINT";
const defaultConfig = {
  applications: {},
  importmap: {},
  layout: { content: { tag: "slot" } },
  settings: {
    "4xx": {
      401: {
        entry: page401Url,
        integrationMode: "parcel"
      },
      404: {
        entry: page404Url,
        integrationMode: "parcel"
      }
    },
    "5xx": {
      500: {
        entry: page500Url,
        integrationMode: "parcel"
      }
    },
    composerUri,
    defaultUrl: "./"
  },
  shared: { properties: {} },
  version: 2
};
const patchApplicationsOntoDocument = (document, applications) => Object.entries(applications).reduce((apps, [key, nextApp]) => {
  if ("route" in nextApp) {
    apps[key] = {
      ...nextApp,
      route: computeAbsoluteRoute(document, nextApp.route)
    };
  }
  if ("config" in nextApp && typeof nextApp.config === "string") {
    apps[key] = {
      ...nextApp,
      config: computeAbsoluteRoute(document, nextApp.config)
    };
  }
  return apps;
}, applications);
function mergeConfig(document, input) {
  var _a, _b, _c, _d, _e, _f, _g, _h, _i, _j, _k, _l, _m, _n, _o, _p;
  const def = defaultConfig;
  const defaultUrl = computeAbsoluteRoute(
    document,
    (_b = (_a = input.settings) == null ? void 0 : _a.defaultUrl) != null ? _b : def.settings.defaultUrl
  );
  const applications = patchApplicationsOntoDocument(
    document,
    (_c = input.applications) != null ? _c : def.applications
  );
  return {
    applications,
    importmap: (_d = input.importmap) != null ? _d : def.importmap,
    layout: {
      ...input.layout,
      content: (_f = (_e = input.layout) == null ? void 0 : _e.content) != null ? _f : def.layout.content
    },
    settings: {
      "4xx": {
        ...def.settings["4xx"],
        ...(_g = input.settings) == null ? void 0 : _g["4xx"]
      },
      "5xx": {
        ...def.settings["5xx"],
        ...(_h = input.settings) == null ? void 0 : _h["5xx"]
      },
      composerUri: (_j = (_i = input.settings) == null ? void 0 : _i.composerUri) != null ? _j : def.settings.composerUri,
      defaultUrl,
      mountPoint: (_l = (_k = input.settings) == null ? void 0 : _k.mountPoint) != null ? _l : def.settings.mountPoint,
      mountPointSelector: (_n = (_m = input.settings) == null ? void 0 : _m.mountPointSelector) != null ? _n : def.settings.mountPointSelector
    },
    shared: {
      ...input.shared,
      properties: {
        ...(_p = (_o = input.shared) == null ? void 0 : _o.properties) != null ? _p : def.shared.properties
      }
    },
    version: 2
  };
}
export {
  MICRO_LC_MOUNT_POINT,
  defaultConfig,
  mergeConfig
};
