/*!

Copyright 2022 Mia srl

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

	http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.

*/
function composeCSSRuleText(rules) {
  return Object.keys(rules).reduce((cssText, ruleName) => {
    const ruleValue = rules[ruleName];
    return cssText.concat("".concat(ruleName, ": ").concat(ruleValue, ";\n"));
  }, "");
}
function composeCSSNodeText(selector, rules) {
  return "\n    ".concat(selector, " {\n      ").concat(composeCSSRuleText(rules), "\n    }\n  ");
}
function composeTextStyleSheet(node) {
  return Object.entries(node).reduce((stylesheet, [selector, rules]) => {
    return stylesheet.concat("".concat(composeCSSNodeText(
      selector,
      rules
    ), "\n"));
  }, "");
}
function composeStyleSheet(node) {
  return Object.entries(node).reduce((stylesheet, [selector, rules]) => {
    stylesheet.insertRule("".concat(composeCSSNodeText(
      selector,
      rules
    ), "\n"));
    return stylesheet;
  }, new CSSStyleSheet());
}
function createCSSStyleSheets({ global, nodes }) {
  const stylesheets = [];
  nodes && stylesheets.push(composeStyleSheet(nodes));
  global && stylesheets.push(composeStyleSheet({ ":host": global }));
  return stylesheets;
}
function injectStyleToElements({ global, nodes }, [globalTag, nodesTag], shadow) {
  const selector = shadow ? ":host" : ":root";
  const elements = [];
  global && elements.push(Object.assign(
    globalTag,
    {
      textContent: composeTextStyleSheet({ [selector]: global })
    }
  ));
  nodes && elements.push(Object.assign(
    nodesTag,
    {
      textContent: composeTextStyleSheet(nodes)
    }
  ));
  return elements;
}
export {
  createCSSStyleSheets,
  injectStyleToElements
};
