/*!

Copyright 2022 Mia srl

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

	http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.

*/
import logger from "../logger/index.js";
import { toArray } from "./array.js";
const jsonAcceptedTypes = [
  "application/json",
  "text/x-json"
];
const yamlAcceptedTypes = [
  "application/yaml",
  "application/x-yaml",
  "text/yaml"
];
const acceptedTypes = [
  ...jsonAcceptedTypes,
  ...yamlAcceptedTypes
];
function isSchemaOptions(input) {
  return Object.prototype.hasOwnProperty.call(input, "id") && Object.prototype.hasOwnProperty.call(input, "parts");
}
async function jsonFetcher(url, init) {
  return globalThis.fetch(
    new URL(url, globalThis.document.baseURI),
    {
      ...init,
      headers: {
        ...init == null ? void 0 : init.headers,
        Accept: acceptedTypes.join(", ")
      }
    }
  ).then((res) => {
    var _a;
    const contentType = (_a = res.headers.get("Content-Type")) != null ? _a : "";
    const isJson = jsonAcceptedTypes.reduce(
      (accepted, str) => contentType.includes(str) || accepted,
      false
    );
    const isYaml = yamlAcceptedTypes.reduce(
      (accepted, str) => contentType.includes(str) || accepted,
      false
    );
    if (res.ok && isJson) {
      return res.json();
    }
    if (res.ok && isYaml) {
      return Promise.all([import("js-yaml"), res.text()]).then(([{ default: yaml }, data]) => {
        return yaml.load(data, { json: true, schema: yaml.JSON_SCHEMA });
      });
    }
    return Promise.reject(new TypeError("20"));
  });
}
async function jsonToObject(input, schema) {
  if (true) {
    if (schema) {
      return Promise.all([
        import("ajv"),
        import("ajv-formats")
      ]).then(([
        { default: Ajv },
        { default: addFormats }
      ]) => {
        try {
          let schemas = toArray(schema);
          let id = schema.$id;
          if (isSchemaOptions(schema)) {
            schemas = schema.parts;
            id = schema.id;
          }
          const ajv = new Ajv({ schemas });
          addFormats(ajv);
          const validate = ajv.getSchema(id);
          validate(input);
          const { errors: cause } = validate;
          if (cause) {
            return Promise.reject(new TypeError("21", { cause }));
          }
        } catch (err) {
          return Promise.reject(new TypeError("22", { cause: err.message }));
        }
        return Promise.resolve(input);
      });
    }
  }
  return Promise.resolve(input);
}
function invalidJsonCatcher(err, data, file) {
  if (err instanceof TypeError) {
    if (err.message === "20") {
      logger.error(err.message, file != null ? file : '"unknown"');
    } else {
      logger.error("1", err.message);
    }
  }
  return data;
}
function jsonToObjectCatcher(err, data, file) {
  if (err instanceof TypeError) {
    const handledCodes = ["21", "22"];
    if (handledCodes.includes(err.message)) {
      logger.error(err.message, file != null ? file : '"unknown"', err.cause ? JSON.stringify(err.cause) : "null");
    } else {
      logger.error(err.message);
    }
  }
  return data;
}
export {
  invalidJsonCatcher,
  jsonFetcher,
  jsonToObject,
  jsonToObjectCatcher
};
