/*!

Copyright 2022 Mia srl

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

	http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.

*/
import { BehaviorSubject } from "rxjs";
import { MFELoader } from "./mfe-loader.js";
import { currentApplication$, getCurrentApplicationAssets } from "./router.js";
function createMicrolcApiInstance() {
  const { microfrontendLoader } = this;
  const currentState = {};
  const bus = new BehaviorSubject(currentState);
  return () => Object.freeze({
    createLoader() {
      return new MFELoader(this.getCurrentConfig(), () => this, microfrontendLoader);
    },
    currentApplication$,
    getApplications: () => Object.freeze({ ...this._config.applications }),
    getCurrentApplication: () => Object.freeze({ ...getCurrentApplicationAssets() }),
    getCurrentConfig: () => Object.freeze({ ...this._config }),
    getExtensions: () => Object.freeze({ ...this._extensions }),
    next: (value) => bus.next(value),
    router: {
      goTo: (url) => {
        if (url !== void 0) {
          const regularUrl = new URL(url, this.ownerDocument.baseURI);
          const isSameOrigin = regularUrl.origin === window.location.origin;
          const isSamePathname = regularUrl.pathname === window.location.pathname;
          if (isSameOrigin && isSamePathname) {
            window.history.replaceState(window.history.state, "", url);
          } else if (isSameOrigin) {
            window.history.pushState(null, "", url);
          } else {
            window.open(url);
          }
        }
      },
      goToApplication: async (_id, data) => {
        var _a;
        if (!(_id in this._config.applications)) {
          return this._rerouteToError(404).then(() => {
          });
        }
        const route = (_a = this.loadedApps.get(_id)) == null ? void 0 : _a[0];
        return this._reroute({ data, url: route }).then(() => {
        });
      },
      goToErrorPage: async (statusCode, reason) => {
        return this._rerouteToError(statusCode, reason).then(() => {
        });
      },
      open: (url, target, features) => window.open(url, target, features),
      pushState: (data, unused, url) => window.history.pushState(data, unused, url),
      replaceState: (data, unused, url) => window.history.replaceState(data, unused, url)
    },
    set: (event) => {
      const newState = Object.assign(currentState, event);
      bus.next(newState);
    },
    setCurrentConfig: (newConfig) => {
      this.config = newConfig;
    },
    setExtension: (key, value) => {
      this._extensions[key] = value;
      return Object.freeze({ ...this._extensions });
    },
    subscribe: (next) => bus.asObservable().subscribe(next)
  });
}
export {
  createMicrolcApiInstance
};
