/*!

Copyright 2022 Mia srl

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

	http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.

*/
import { createCSSStyleSheets, injectStyleToElements } from "../../dom-manipulation/index.js";
import { jsonFetcher, jsonToObject, jsonToObjectCatcher } from "../../utils/json.js";
import { currentApplication$, rerouteErrorHandler } from "./router.js";
import { handleUpdateError } from "./update.js";
function updateCSS(css) {
  this._styleElements.forEach((style) => {
    style.remove();
  });
  if (this.shadowRoot && "adoptedStyleSheets" in this.ownerDocument) {
    const stylesheets = createCSSStyleSheets(css);
    this.shadowRoot.adoptedStyleSheets = [...stylesheets];
  } else {
    const styleTags = Array(2).fill(0).map(
      () => this.ownerDocument.createElement("style")
    );
    this._styleElements = injectStyleToElements(css, styleTags, this.shadowRoot !== null);
    this._styleElements.forEach((el) => {
      this.shadowRoot ? this.shadowRoot.insertBefore(el, this.shadowRoot.firstChild) : this.ownerDocument.head.appendChild(el);
    });
  }
}
function initLanguageExtension() {
  let currentLanguage = window.navigator.language;
  return {
    getFallbackLanguage: () => {
      return this._fallbackLanguage;
    },
    getLanguage() {
      return currentLanguage;
    },
    setLanguage: (nextLang) => {
      currentLanguage = nextLang;
      setTimeout(() => {
        this._prepareForUpdate();
        this.update().then((done) => {
          if (done) {
            this.matchCache.invalidateCache();
            this._reroute().catch(rerouteErrorHandler);
            this._completeUpdate();
          }
        }).catch((err) => handleUpdateError(currentApplication$, err));
      });
    }
  };
}
function initBaseExtensions() {
  return {
    css: {
      setStyle: (css) => {
        updateCSS.call(this, css);
      }
    },
    head: {
      setIcon: ({ href, sizes, type }) => {
        var _a;
        const icon = (_a = this.ownerDocument.head.querySelector("link[rel=icon]")) != null ? _a : this.ownerDocument.createElement("link");
        Object.assign(icon, {
          href: href != null ? href : icon.href,
          sizes: sizes != null ? sizes : icon.sizes,
          type: type != null ? type : icon.type
        });
      },
      setTitle: (nextTitle) => {
        var _a;
        const title = (_a = this.ownerDocument.head.querySelector("title")) != null ? _a : this.ownerDocument.createElement("title");
        title.textContent = nextTitle;
        !title.isConnected && this.ownerDocument.head.appendChild(title);
      }
    },
    httpClient: (input, init) => window.fetch(input, init),
    json: {
      fetcher: jsonFetcher,
      async validator(json, schema, {
        defaultValue = json,
        file
      } = {}) {
        return jsonToObject(json, schema).catch(
          (err) => jsonToObjectCatcher(err, defaultValue, file)
        );
      }
    },
    language: initLanguageExtension.call(this)
  };
}
export {
  initBaseExtensions,
  updateCSS
};
