/*!

Copyright 2022 Mia srl

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

	http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.

*/
var __defProp = Object.defineProperty;
var __defNormalProp = (obj, key, value) => key in obj ? __defProp(obj, key, { enumerable: true, configurable: true, writable: true, value }) : obj[key] = value;
var __publicField = (obj, key, value) => {
  __defNormalProp(obj, typeof key !== "symbol" ? key + "" : key, value);
  return value;
};
import { BehaviorSubject } from "rxjs";
import logger from "../../logger/index.js";
import { getPublicPath, errorMap, RoutingErrorMessage, postProcessTemplate, microlcFetch } from "./handler.js";
import { effectiveRouteLength, urlMatch } from "./url-matcher.js";
let currentApplication;
let currentApplicationBus = new BehaviorSubject(void 0);
let applicationHandlers = /* @__PURE__ */ new Map();
const currentApplication$ = currentApplicationBus.asObservable();
function getMount(name, shouldMountByLoading = false) {
  const app = name && applicationHandlers.get(name);
  if (app) {
    return () => app.loadPromise.then(() => app.bootstrapPromise).then(() => app.getStatus() === "NOT_MOUNTED" && !shouldMountByLoading ? app.mount() : app.mountPromise);
  }
}
function getUnmount(name = currentApplication) {
  const app = name && applicationHandlers.get(name);
  if (app) {
    if (app.getStatus() !== "UNMOUNTING") {
      return () => app.loadPromise.then(() => app.bootstrapPromise).then(() => app.mountPromise).then(() => app.getStatus() === "NOT_MOUNTED" ? app.unmountPromise : app.unmount());
    }
    return () => app.unmountPromise;
  }
}
function getUpdate(name = currentApplication) {
  let update;
  if (name) {
    const app = applicationHandlers.get(name);
    if (app) {
      update = (props) => app.loadPromise.then(() => app.bootstrapPromise).then(() => app.mountPromise).then(() => {
        var _a;
        return (_a = app.update) == null ? void 0 : _a.call(app, props);
      });
    }
  }
  return update;
}
function getCurrentApplicationAssets() {
  if (currentApplication === void 0) {
    return;
  }
  const handlers = applicationHandlers.get(currentApplication);
  if (handlers === void 0) {
    return;
  }
  return {
    handlers: applicationHandlers.get(currentApplication),
    id: currentApplication
  };
}
class MatchCache extends Map {
  constructor() {
    super(...arguments);
    __publicField(this, "_default");
  }
  setDefault(match) {
    this._default = [void 0, void 0, match];
  }
  getDefault() {
    return this._default;
  }
  invalidateCache() {
    this._default = void 0;
    this.clear();
  }
}
function rerouteErrorHandler(err) {
  logger.error("51", err.message);
  return null;
}
function getIFramePathname(win = window, baseURI) {
  let completeHref = window.location.href;
  const { location: { origin, href } } = win;
  const iframePathname = href.match(new RegExp(origin));
  if ((iframePathname == null ? void 0 : iframePathname.index) !== void 0) {
    completeHref = href.slice(iframePathname.index + origin.length);
  }
  return new URL(completeHref, baseURI);
}
const updateCountersAndResults = (pathname, route, defaultUrl, baseURI, args, counters, result) => {
  const { pathname: routePathname } = new URL(route, baseURI);
  const exact = urlMatch(pathname, routePathname);
  const routePathnameLength = effectiveRouteLength(routePathname);
  if (exact && routePathnameLength > counters[0]) {
    counters[0] = routePathnameLength;
    result[0] = args;
  }
  const { pathname: routeWtsPathname } = new URL(route.replace(/\/$/, ""), baseURI);
  const wts = urlMatch(pathname, routeWtsPathname);
  const routeWtsPathnameLength = effectiveRouteLength(routeWtsPathname);
  if (wts && routeWtsPathnameLength > counters[1]) {
    counters[1] = routeWtsPathnameLength;
    result[1] = args;
  }
  const { pathname: defaultPathname } = new URL(defaultUrl, baseURI);
  const def = urlMatch(defaultPathname, routePathname);
  if (def && routePathnameLength > counters[2]) {
    counters[2] = routePathnameLength;
    result[2] = args;
  }
};
function getNextMatchingRoute(url) {
  const {
    config: {
      settings: {
        defaultUrl
      }
    },
    ownerDocument: {
      baseURI
    }
  } = this;
  const result = Array(3).fill(void 0);
  const counters = Array(3).fill(0);
  const { pathname } = url ? new URL(url, baseURI) : getIFramePathname(window, baseURI);
  for (const [route, args] of this.loadedApps.values()) {
    if (route === void 0) {
      continue;
    }
    updateCountersAndResults(
      pathname,
      route,
      defaultUrl,
      baseURI,
      args,
      counters,
      result
    );
  }
  url !== void 0 ? this.matchCache.set(url, result) : this.matchCache.setDefault(result[2]);
  return {
    counters,
    result
  };
}
function isSameError(first, second) {
  if (first.message !== second.message) {
    return false;
  }
  if (first.reason !== second.reason) {
    return false;
  }
  if (first.status !== second.status) {
    return false;
  }
  return true;
}
async function flushAndGo(nextMatch, args = {}, incomingError, previousIncomingError) {
  var _a, _b;
  const recursiveFlushAndGo = flushAndGo.bind(this);
  if (previousIncomingError && incomingError && isSameError(previousIncomingError, incomingError)) {
    return void 0;
  }
  let error = incomingError;
  const unmount = getUnmount();
  currentApplication = nextMatch.name;
  await (unmount == null ? void 0 : unmount().catch(rerouteErrorHandler));
  const { url, method = "push", data } = args != null ? args : {};
  if (url !== void 0) {
    if (method === "push") {
      window.history.pushState(data, "", url);
    } else {
      window.history.replaceState(data, "", url);
    }
  }
  let handlers = applicationHandlers.get(nextMatch.name);
  const shouldMountByLoading = handlers === void 0;
  if (shouldMountByLoading) {
    const appConfigName = this.applicationMapping.get(currentApplication);
    const [route] = (_a = appConfigName !== void 0 ? this.loadedApps.get(appConfigName) : appConfigName) != null ? _a : [];
    const application = nextMatch;
    handlers = this.microfrontendLoader.loadMicroApp(application, {
      fetch: (input, init) => microlcFetch(input, init, error).then(([res, outgoingError]) => {
        error = outgoingError;
        return res;
      }),
      getPublicPath,
      postProcessTemplate: (tplResult) => {
        var _a2, _b2;
        return postProcessTemplate(tplResult, {
          baseURI: this.ownerDocument.baseURI,
          injectBase: (_a2 = nextMatch.props) == null ? void 0 : _a2.injectBase,
          name: nextMatch.name,
          url: (_b2 = route != null ? route : url) != null ? _b2 : window.location.href
        });
      },
      sandbox: { speedy: false }
    });
    applicationHandlers.set(nextMatch.name, handlers);
  }
  if (error === void 0) {
    currentApplication && currentApplicationBus.next(
      this.applicationMapping.get(currentApplication)
    );
  } else if (error.message === RoutingErrorMessage.NOT_FOUND) {
    currentApplicationBus.next(void 0);
  }
  await ((_b = getMount(currentApplication, shouldMountByLoading)) == null ? void 0 : _b().catch(async (err) => {
    var _a2, _b2, _c;
    const message = (_a2 = error == null ? void 0 : error.message) != null ? _a2 : RoutingErrorMessage.INTERNAL_SERVER_ERROR;
    const status = (_b2 = error == null ? void 0 : error.status) != null ? _b2 : 500;
    const reason = err instanceof TypeError ? err.message : void 0;
    const errorRoute = (_c = this.loadedApps.get("".concat(this.instance, "-").concat(status))) == null ? void 0 : _c[1];
    error = { ...error, message, status };
    if (errorRoute) {
      await recursiveFlushAndGo(errorRoute, args, error, incomingError).then((update) => update == null ? void 0 : update({ message, reason }));
    }
    return void 0;
  }));
  return getUpdate();
}
async function rerouteToError(statusCode, reason) {
  var _a;
  const code = statusCode != null ? statusCode : 404;
  const nextMatch = (_a = this.loadedApps.get("".concat(this.instance, "-").concat(code))) == null ? void 0 : _a[1];
  if (!nextMatch) {
    return Promise.reject(new TypeError("no ".concat(code, " page available")));
  }
  const recursiveFlushAndGo = flushAndGo.bind(this);
  return recursiveFlushAndGo(nextMatch, {}, { message: errorMap[code], reason, status: code }).then(async (update) => {
    await (update == null ? void 0 : update({ message: "Oops! Something went wrong", reason }));
    return void 0;
  });
}
function isDefault(url, baseURI) {
  const { pathname } = new URL(url, window.document.baseURI);
  return pathname.replace(/\/$/, "") === new URL(baseURI, window.document.baseURI).pathname.replace(/\/$/, "");
}
async function reroute(args = {}) {
  var _a;
  let { url, data, method = "push" } = args;
  let matchingResults = Array(3).fill(void 0);
  let counters = Array(3).fill(0);
  const isDefaultCached = this.matchCache.getDefault();
  const cachedMatch = url !== void 0 ? this.matchCache.get(url) : void 0;
  if (url === void 0 && isDefaultCached) {
    matchingResults = isDefaultCached;
  } else if (cachedMatch) {
    matchingResults = cachedMatch;
  } else {
    const recursiveGetNextMatchingRoute = getNextMatchingRoute.bind(this);
    const nextMatch2 = recursiveGetNextMatchingRoute(url);
    matchingResults = nextMatch2.result;
    counters = nextMatch2.counters;
  }
  const [exactMatch, nextMatchWithTrailingSlash, defaultMatch] = matchingResults;
  let nextMatch = exactMatch;
  let error;
  if (nextMatchWithTrailingSlash && counters[1] > counters[0] || !exactMatch && nextMatchWithTrailingSlash) {
    const route = this.loadedRoutes.get(nextMatchWithTrailingSlash.name);
    const recursiveReroute = reroute.bind(this);
    return recursiveReroute({ data: window.history.state, method: "replace", url: route });
  }
  if (!exactMatch && isDefault(url != null ? url : window.location.href, this.ownerDocument.baseURI)) {
    url = this.config.settings.defaultUrl;
    data = window.history.state;
    method = "replace";
    nextMatch = defaultMatch;
  }
  if (!nextMatch) {
    error = { message: RoutingErrorMessage.NOT_FOUND, reason: "Page ".concat(url != null ? url : window.location.pathname, " cannot be found"), status: 404 };
    nextMatch = (_a = this.loadedApps.get("".concat(this.instance, "-404"))) == null ? void 0 : _a[1];
  }
  if (!nextMatch) {
    error = { message: RoutingErrorMessage.NOT_FOUND, reason: "No 404 page available", status: 404 };
    return Promise.reject(new TypeError("no 404 page available"));
  }
  const recursiveFlushAndGo = flushAndGo.bind(this);
  return nextMatch.name !== currentApplication ? recursiveFlushAndGo(nextMatch, { data, method, url }, error).then(async (update) => {
    await (update == null ? void 0 : update({ ...error }));
  }) : Promise.resolve();
}
let popstate;
let domcontent;
function popStateListener(event) {
  var _a;
  const target = (_a = event.target) != null ? _a : window;
  const recursiveReroute = reroute.bind(this);
  recursiveReroute({ data: target.history.state, url: target.location.href }).catch(rerouteErrorHandler);
}
function domContentLoaded(event) {
  console.warn("[micro-lc] unhandled DOMContentLoaded event", event);
}
function createRouter() {
  popstate = popStateListener.bind(this);
  domcontent = domContentLoaded.bind(window);
  window.addEventListener("popstate", popstate);
  window.addEventListener("DOMContentLoaded", domContentLoaded);
}
function removeRouter() {
  currentApplication = void 0;
  currentApplicationBus = new BehaviorSubject(void 0);
  applicationHandlers = /* @__PURE__ */ new Map();
  popstate && window.removeEventListener("popstate", popstate);
  domcontent && window.removeEventListener("DOMContentLoaded", domcontent);
}
export {
  MatchCache,
  createRouter,
  currentApplication$,
  getCurrentApplicationAssets,
  getUnmount,
  removeRouter,
  reroute,
  rerouteErrorHandler,
  rerouteToError
};
