/*!

Copyright 2022 Mia srl

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

	http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.

*/
import { take } from "rxjs";
import { defaultConfig } from "../../config";
import logger from "../../logger";
import { invalidJsonCatcher, jsonFetcher, jsonToObject, jsonToObjectCatcher } from "../../utils/json";
import { prepareApplicationConfig } from "./mfe-loader";
const defaultInitOptions = {
  mapOverrides: true,
  shimMode: true
};
const COMPOSER_BODY_CLASS = "composer-body";
async function initImportMapSupport() {
  const { esmsInitOptions } = window;
  !esmsInitOptions && Object.defineProperty(window, "esmsInitOptions", {
    configurable: true,
    value: defaultInitOptions,
    writable: true
  });
  const done = Promise.resolve();
  if (!("importShim" in window)) {
    await import("es-module-shims").catch(
      logger.dynamicImportError("es-module-shims")
    );
  }
  return done;
}
const handleInitImportMapError = (err) => {
  logger.error("2", err.message);
};
const handleUpdateError = (currentApplication$, err) => {
  currentApplication$.pipe(take(1)).subscribe((app) => {
    logger.error("50", app != null ? app : "[unknown]", err.message);
  });
};
async function fetchConfig(url, init) {
  return jsonFetcher(url, init).then((json) => {
    if (true) {
      return Promise.all([
        Promise.resolve(json),
        import("../../utils/schemas.js").then((schemas) => ({
          id: schemas.configSchema.$id,
          parts: schemas
        }))
      ]);
    }
    return [json, void 0];
  }).then(
    ([json, schema]) => jsonToObject(json, schema).catch(
      (err) => jsonToObjectCatcher(
        err,
        defaultConfig,
        '"micro-lc config"'
      )
    )
  ).catch(
    (err) => invalidJsonCatcher(
      err,
      defaultConfig,
      '"micro-lc config"'
    )
  );
}
async function updateGlobalImportMap(importmap) {
  importShim.addImportMap(importmap);
  return Promise.resolve();
}
async function getApplicationSchema() {
  let schema;
  if (true) {
    schema = await import("../../utils/schemas.js").then((schemas) => ({
      id: schemas.pluginSchema.$id,
      parts: schemas
    }));
  }
  return schema;
}
function getContainer(selector) {
  const selected = selector ? this.querySelector(selector) : this._container;
  return selected != null ? selected : this._container;
}
async function updateApplications() {
  const {
    _config: {
      settings: {
        "4xx": pages4xx,
        "5xx": pages5xx,
        composerUri,
        mountPointSelector
      },
      applications,
      shared: {
        properties: sharedProperties
      }
    }
  } = this;
  const errorPages = [
    ...Object.entries(pages4xx),
    ...Object.entries(pages5xx)
  ];
  const pages = [
    ...errorPages,
    ...Object.entries(applications)
  ];
  pages.reduce((acc, [id, app], idx) => {
    const { name, makeConfig } = prepareApplicationConfig(id, app, composerUri);
    const container = getContainer.call(this, mountPointSelector);
    let idScopedByInstance = id;
    if (idx < errorPages.length) {
      idScopedByInstance = "".concat(this._instance, "-").concat(id);
    } else if (app.route !== void 0) {
      acc.routes.set(name, app.route);
    }
    acc.mapping.set(name, idScopedByInstance);
    acc.apps.set(idScopedByInstance, [app.route, makeConfig(container, this.getApi(), sharedProperties)]);
    return acc;
  }, {
    apps: this.loadedApps,
    mapping: this.applicationMapping,
    routes: this.loadedRoutes
  });
  return Promise.resolve();
}
export {
  COMPOSER_BODY_CLASS,
  fetchConfig,
  getApplicationSchema,
  handleInitImportMapError,
  handleUpdateError,
  initImportMapSupport,
  updateApplications,
  updateGlobalImportMap
};
