/*!

Copyright 2022 Mia srl

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

	http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.

*/
var __defProp = Object.defineProperty;
var __defNormalProp = (obj, key, value) => key in obj ? __defProp(obj, key, { enumerable: true, configurable: true, writable: true, value }) : obj[key] = value;
var __publicField = (obj, key, value) => {
  __defNormalProp(obj, typeof key !== "symbol" ? key + "" : key, value);
  return value;
};
import { createComposerContext, premount } from "@micro-lc/composer";
import { camelCase, kebabCase } from "lodash-es";
import { mergeConfig, defaultConfig } from "../config.js";
import { craftLanguageHeader } from "../utils/lang.js";
import {
  COMPOSER_BODY_CLASS,
  handleUpdateError,
  getUnmount,
  currentApplication$,
  rerouteToError,
  MatchCache,
  createMicrolcApiInstance,
  createRouter,
  removeRouter,
  reroute,
  rerouteErrorHandler,
  fetchConfig,
  handleInitImportMapError,
  initImportMapSupport,
  updateApplications,
  updateGlobalImportMap,
  initBaseExtensions,
  createQiankunInstance
} from "./lib/index.js";
class Microlc extends HTMLElement {
  constructor() {
    var _a, _b;
    super();
    __publicField(this, "_wasDisconnected", false);
    __publicField(this, "_updateComplete", true);
    __publicField(this, "_updateRequests", 0);
    __publicField(this, "_$$updatesCount", null);
    __publicField(this, "_instance", window.crypto.randomUUID());
    __publicField(this, "_config");
    __publicField(this, "_configSrc");
    __publicField(this, "_fallbackLanguage");
    __publicField(this, "_disableShadowDom");
    __publicField(this, "_reroute", reroute.bind(this));
    __publicField(this, "_rerouteToError", rerouteToError.bind(this));
    __publicField(this, "_microfrontendLoader", createQiankunInstance());
    // queries
    __publicField(this, "_styleElements", []);
    // 🚥 router
    __publicField(this, "loadedApps", /* @__PURE__ */ new Map());
    __publicField(this, "loadedRoutes", /* @__PURE__ */ new Map());
    __publicField(this, "applicationMapping", /* @__PURE__ */ new Map());
    __publicField(this, "matchCache", new MatchCache());
    // 🐝 api
    __publicField(this, "_extensions", initBaseExtensions.call(this));
    __publicField(this, "getApi", createMicrolcApiInstance.call(this));
    // DOM
    __publicField(this, "_style");
    __publicField(this, "_container");
    const shadow = this.getAttribute("disable-shadow-dom") === null;
    const microlcRootId = (_a = this.getAttribute("root-id")) != null ? _a : "__micro_lc";
    if (shadow) {
      this.attachShadow({ mode: "open" });
    }
    const appendStyle = this.getAttribute("disable-styling") === null;
    this._style = Object.assign(
      this.ownerDocument.createElement("style"),
      {
        textContent: "\n          div#__micro_lc {\n            width: 100%;\n            height: 100%;\n          }\n\n          div#__micro_lc > :first-child {\n            width: inherit;\n            height: inherit;\n            overflow: hidden\n          }\n\n          div#__micro_lc > :first-child > div.".concat(COMPOSER_BODY_CLASS, " {\n            width: inherit;\n            height: inherit;\n            overflow: hidden\n          }\n        ")
      }
    );
    appendStyle && this.appendChild(this._style);
    this._container = this.ownerDocument.createElement("div");
    this._container.setAttribute("id", microlcRootId);
    this.appendChild(this._container);
    this._handlePropertyChange("config", (_b = this.config) != null ? _b : defaultConfig);
    this.configSrc !== void 0 && this._handlePropertyChange("configSrc", this.configSrc);
  }
  static get observedAttributes() {
    return ["config-src", "fallback-language"];
  }
  // properties/attributes update
  _prepareForUpdate() {
    this._updateRequests += 1;
    this._updateComplete = false;
  }
  _completeUpdate() {
    this._updateRequests = 0;
    this._updateComplete = true;
  }
  _handlePropertyChange(name, value) {
    switch (name) {
      case "config":
        this._configSrc = null;
        this.removeAttribute("config-src");
        this._prepareForUpdate();
        this._handlePropertyUpdate(
          "config",
          mergeConfig(this.ownerDocument, value)
        );
        break;
      case "configSrc":
        if (value !== this._configSrc) {
          this._prepareForUpdate();
          this._handlePropertyUpdate(
            "configSrc",
            value,
            (input) => typeof input === "string"
          );
        }
        break;
      case "fallbackLanguage":
        if (value !== this._fallbackLanguage) {
          this._prepareForUpdate();
          this._handlePropertyUpdate(
            "fallbackLanguage",
            value,
            (input) => typeof input === "string"
          );
        }
        break;
      default:
        break;
    }
  }
  _handlePropertyUpdate(name, newValue, checkType = (_) => true) {
    const nextValue = checkType(newValue) ? newValue : void 0;
    Object.assign(this, { ["_".concat(name)]: nextValue });
    const candidateAttribute = kebabCase(name);
    if (Microlc.observedAttributes.includes(candidateAttribute)) {
      if (typeof nextValue === "string") {
        this.setAttribute(kebabCase(name), nextValue);
      } else if (nextValue === true) {
        this.setAttribute(kebabCase(name), "");
      } else {
        this.removeAttribute(kebabCase(name));
      }
    }
    setTimeout(() => {
      this.update().then((done) => {
        if (done) {
          this.matchCache.invalidateCache();
          return this._reroute().then(() => done).catch(rerouteErrorHandler);
        }
      }).then((done) => {
        var _a;
        if (done) {
          (_a = this.onload) == null ? void 0 : _a.call(window, new Event("load"));
          this._completeUpdate();
        }
      }).catch((err) => handleUpdateError(currentApplication$, err));
    });
  }
  get config() {
    return this._config;
  }
  set config(src) {
    this._handlePropertyChange("config", src);
  }
  /**
   * @observedProperty --> mirrored by `config-src`
   */
  get configSrc() {
    return this._configSrc;
  }
  set configSrc(src) {
    this._handlePropertyChange("configSrc", src);
  }
  /**
   * @observedProperty --> mirrored by `fallback-language`
   */
  get fallbackLanguage() {
    return this._fallbackLanguage;
  }
  set fallbackLanguage(lang) {
    this._handlePropertyChange("fallbackLanguage", lang);
  }
  get updateComplete() {
    return this._updateComplete;
  }
  get $$updatesCount() {
    return this._$$updatesCount;
  }
  get microfrontendLoader() {
    return this._microfrontendLoader;
  }
  get instance() {
    return this._instance;
  }
  connectedCallback() {
    if (this._wasDisconnected && this._disableShadowDom) {
      this._styleElements.forEach((element) => {
        this.ownerDocument.head.appendChild(element);
      });
    }
    createRouter.call(this);
    this._wasDisconnected = false;
  }
  attributeChangedCallback(name, oldValue, newValue) {
    let nextValue = newValue;
    if (newValue === "") {
      nextValue = null;
    }
    if (oldValue !== nextValue) {
      Object.assign(this, { [camelCase(name)]: nextValue });
    }
  }
  async update() {
    var _a, _b;
    if (this._updateRequests === 1) {
      if (false) {
        if (this._$$updatesCount === null) {
          this._$$updatesCount = 0;
        }
        this._$$updatesCount += 1;
      }
      if (typeof this._configSrc === "string") {
        const headers = craftLanguageHeader(
          (_a = this.getApi().getExtensions().language) == null ? void 0 : _a.getLanguage(),
          (_b = this.getApi().getExtensions().language) == null ? void 0 : _b.getFallbackLanguage()
        );
        const config = await fetchConfig(this._configSrc, { headers });
        this._config = mergeConfig(this.ownerDocument, config);
      }
      await initImportMapSupport().catch(handleInitImportMapError);
      await updateGlobalImportMap(this._config.importmap);
      await this.render();
      await updateApplications.call(this);
      return Promise.resolve(true);
    }
    this._updateRequests -= 1;
    return Promise.resolve(false);
  }
  async render() {
    const {
      _config: {
        layout,
        settings: {
          mountPoint,
          mountPointSelector
        },
        shared: {
          properties
        }
      }
    } = this;
    const unmount = getUnmount();
    await (unmount == null ? void 0 : unmount().catch(rerouteErrorHandler));
    if (this.shadowRoot) {
      Array.from(this.shadowRoot.children).forEach((child) => {
        child.remove();
      });
    }
    Array.from(this._container.children).forEach((child) => {
      child.remove();
    });
    const { content } = await premount(layout);
    const composerByContent = async (conf) => createComposerContext(conf, {
      context: { composerApi: { createComposerContext, premount }, microlcApi: this.getApi(), ...properties },
      extraProperties: /* @__PURE__ */ new Set(["microlcApi", "composerApi", ...Object.keys(properties)])
    });
    const layoutAppender = await composerByContent(content);
    if (this.shadowRoot) {
      layoutAppender(this.shadowRoot);
    }
    if (mountPoint) {
      let isDomMountable = false;
      if (true) {
        if (mountPointSelector !== void 0) {
          const mountPointAppender = await composerByContent(mountPoint);
          const temporaryMountPoint = this.ownerDocument.createElement("div");
          mountPointAppender(temporaryMountPoint);
          isDomMountable = temporaryMountPoint.querySelector(mountPointSelector) !== null;
        }
      } else {
        isDomMountable = true;
      }
      if (isDomMountable) {
        const mountPointAppender = await composerByContent(mountPoint);
        mountPointAppender(this._container);
      }
    }
  }
  disconnectedCallback() {
    this._styleElements.forEach((el) => {
      el.remove();
    });
    removeRouter.call(this);
    this._wasDisconnected = true;
  }
}
export {
  Microlc
};
