/**
  Copyright 2022 Mia srl

  Licensed under the Apache License, Version 2.0 (the "License");
  you may not use this file except in compliance with the License.
  You may obtain a copy of the License at

      http://www.apache.org/licenses/LICENSE-2.0

  Unless required by applicable law or agreed to in writing, software
  distributed under the License is distributed on an "AS IS" BASIS,
  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
  See the License for the specific language governing permissions and
  limitations under the License.
*/
import type { Observable, Subscription } from 'rxjs';
import type { CompleteConfig } from '../../config';
import type { Microlc } from '../micro-lc';
import type { BaseExtension } from './extensions';
import { MFELoader } from './mfe-loader.js';
import type { RoutelessMicroApp } from './qiankun';
export type MicrolcEvent = Record<string, unknown>;
export interface MicrolcApi<T extends BaseExtension = BaseExtension, E extends MicrolcEvent = MicrolcEvent> {
    readonly createLoader: () => MFELoader<T, E> | undefined;
    readonly currentApplication$: Observable<string | undefined>;
    readonly getApplications: () => Readonly<CompleteConfig['applications']>;
    readonly getCurrentApplication: () => Readonly<Partial<{
        handlers: RoutelessMicroApp | undefined;
        id: string;
    }>>;
    readonly getCurrentConfig: () => Readonly<CompleteConfig>;
    /**
     * @returns a frozen object containing all set extension. Can be modified by `setExtension` api method
     */
    readonly getExtensions: () => Readonly<Partial<T>>;
    /**
     * @param value {Partial<E>} next value to be pushed in the current state bus
     * @returns
     */
    readonly next: (value: E) => void;
    readonly router: {
        goTo: (url: string | URL | undefined) => void;
        goToApplication<S = unknown>(id: string, data?: S): Promise<void>;
        goToErrorPage(statusCode?: number): void;
        open: typeof window.open;
        pushState: typeof window.history.pushState;
        replaceState: typeof window.history.replaceState;
    };
    /**
     * @deprecated
     * @param event {Partial<E>} new state to be embedded with `micro-lc` and sent to the bus
     * @returns
     */
    readonly set: (event: Partial<E>) => void;
    readonly setCurrentConfig: (newConfig: CompleteConfig) => void;
    /**
     * Adds an extension to `micro-lc` like a Redux store, a language
     * context, a proxied fetch/http client, and so on...
     * @example
     * import { createStore } from 'redux'
     * const store = createStore(todos, ['Use Redux'])
     *
     * var api: MicrolcApi<BaseExtension & {store?: typeof store}> // 👈 `micro-lc` api
     * api.setExtension('store', store)
     *
     * @param key {keyof T} the name of the extension of type T
     * @param value {T[keyof T]} the value of the extension
     * @returns a frozen object containing all set extension
     */
    readonly setExtension: (key: keyof T, value: T[keyof T]) => Readonly<T>;
    readonly subscribe: (next: (value: Partial<E>) => void) => Subscription;
}
export declare function createMicrolcApiInstance<T extends BaseExtension, E extends MicrolcEvent>(this: Microlc<T, E>): () => MicrolcApi<T, E>;
