/**
  Copyright 2022 Mia srl

  Licensed under the Apache License, Version 2.0 (the "License");
  you may not use this file except in compliance with the License.
  You may obtain a copy of the License at

      http://www.apache.org/licenses/LICENSE-2.0

  Unless required by applicable law or agreed to in writing, software
  distributed under the License is distributed on an "AS IS" BASIS,
  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
  See the License for the specific language governing permissions and
  limitations under the License.
*/
import type { CSSConfig } from '../../dom-manipulation';
import type { SchemaOptions } from '../../utils/json';
import { jsonFetcher } from '../../utils/json.js';
import type { Microlc } from '../micro-lc';
import type { MicrolcEvent } from './api';
type HTTPClient = typeof window.fetch;
interface JsonCatcherOptions<S> {
    defaultValue?: S;
    file?: string;
}
export type BaseExtension = Record<string, unknown> & {
    css: {
        setStyle: (styles: CSSConfig) => void;
    };
    head: {
        setIcon: (attrs: Partial<Pick<HTMLLinkElement, 'sizes' | 'href' | 'type'>>) => void;
        setTitle: (title: string) => void;
    };
    httpClient: HTTPClient;
    json: {
        fetcher: typeof jsonFetcher;
        validator: <S>(json: unknown, schema: SchemaOptions, opts?: JsonCatcherOptions<S>) => Promise<S>;
    };
    language: {
        getFallbackLanguage: () => string | null | undefined;
        getLanguage: () => string;
        setLanguage: (lang: string) => void;
    };
};
export declare function updateCSS<T extends BaseExtension, E extends MicrolcEvent>(this: Microlc<T, E>, css: CSSConfig): void;
export declare function initBaseExtensions<T extends BaseExtension, E extends MicrolcEvent>(this: Microlc<T, E>): T;
export {};
