import type { Application, ComposableApplication, IFrameApplication, ParcelApplication } from '@micro-lc/interfaces/schemas/v2';
import type { LoadableApp } from 'qiankun';
import type { CompleteConfig } from '../../config';
import type { MicrolcApi, MicrolcEvent } from './api';
import type { BaseExtension } from './extensions';
import type { LoaderApi, LoaderConfiguration, LoaderLifeCycles, RoutelessMicroApp } from './qiankun';
import type { ComposableApplicationProperties } from './update';
type RoutelessApplication = Omit<IFrameApplication, 'route'> | Omit<ComposableApplication, 'route'> | Omit<ParcelApplication, 'route'>;
type LoadableAppContext<T extends BaseExtension = BaseExtension, E extends MicrolcEvent = MicrolcEvent> = LoadableApp<ComposableApplicationProperties<T, E>>;
declare const prepareApplicationConfig: <T extends BaseExtension, E extends MicrolcEvent>(id: string, app: Application | RoutelessApplication, composerUri: string) => {
    makeConfig: (container: HTMLElement | ShadowRoot, microlcApi: MicrolcApi<T, E>, sharedProperties: Record<string, unknown>) => LoadableAppContext<T, E>;
    name: string;
};
declare class MFELoader<T extends BaseExtension = BaseExtension, E extends MicrolcEvent = MicrolcEvent> {
    static ID: string;
    static COMPOSER_BODY_CLASS: string;
    private getApi;
    private loader;
    private composerUri;
    private sharedProperties;
    private handlers;
    private getMount;
    private getUnmount;
    private getUpdate;
    constructor(config: CompleteConfig, getApi: () => MicrolcApi<T, E>, loader: LoaderApi);
    load(app: RoutelessApplication, container: HTMLElement | ShadowRoot): Promise<((props: Record<string, unknown>) => Promise<null>) | undefined>;
    unload(): Promise<null | undefined>;
}
/**
 * Loader meant for external usage.
 *
 * By hiding `qiankun` complexity behind a wrapper allows to mount
 * a microfrontend application within a given DOM context
 * @param name {string} the name of the application
 * @param app {RoutelessApplication} context and integration mode of the application
 * @param container {HTMLElement | ShadowRoot} DOM container
 * @param lifeCycles {LoaderLifeCycles} before/after mount/unmount hooks
 * @param configuration {LoaderConfiguration} setup for sandboxing and encapsulation
 * @param loader {LoaderApi} override loader
 * @returns {RoutelessMicroApp} the context and handlers for a microfrontend application ready for mount
 */
declare const loadApp: (name: string, app: RoutelessApplication & {
    properties?: Record<string, unknown>;
}, container: HTMLElement | ShadowRoot, lifeCycles: LoaderLifeCycles, { composerUri, ...configuration }: LoaderConfiguration, loader?: LoaderApi) => RoutelessMicroApp;
export type { LoadableAppContext, RoutelessApplication };
export { MFELoader, loadApp, prepareApplicationConfig };
