import type { CompleteConfig } from '../config';
import type { MicrolcApi, BaseExtension, RouterContainer, LoaderApi, MicrolcEvent, LoadableAppContext, PushArgs } from './lib';
import { rerouteToError, MatchCache, reroute } from './lib/index.js';
type ObservedAttributes = 'config-src' | 'fallback-language';
type ObservedProperties = 'config' | 'configSrc' | 'fallbackLanguage';
export declare class Microlc<T extends BaseExtension = BaseExtension, E extends MicrolcEvent = MicrolcEvent> extends HTMLElement implements RouterContainer<T, E> {
    static get observedAttributes(): string[];
    private _wasDisconnected;
    private _updateComplete;
    protected _updateRequests: number;
    protected _$$updatesCount: number | null;
    protected _instance: `${string}-${string}-${string}-${string}-${string}`;
    protected _config: CompleteConfig;
    protected _configSrc: string | null | undefined;
    protected _fallbackLanguage: string | null | undefined;
    protected _disableShadowDom: boolean | undefined;
    protected _reroute: (args?: PushArgs) => ReturnType<typeof reroute>;
    protected _rerouteToError: (statusCode?: number | undefined, reason?: string | undefined) => ReturnType<typeof rerouteToError>;
    protected _microfrontendLoader: LoaderApi;
    protected _styleElements: HTMLStyleElement[];
    protected _prepareForUpdate(): void;
    protected _completeUpdate(): void;
    protected _handlePropertyChange(name: ObservedProperties, value: unknown): void;
    protected _handlePropertyUpdate<S>(name: ObservedProperties, newValue: unknown, checkType?: (input: unknown) => input is Exclude<S, undefined>): void;
    get config(): CompleteConfig;
    set config(src: unknown);
    /**
     * @observedProperty --> mirrored by `config-src`
     */
    get configSrc(): string | null | undefined;
    set configSrc(src: unknown);
    /**
     * @observedProperty --> mirrored by `fallback-language`
     */
    get fallbackLanguage(): string | null | undefined;
    set fallbackLanguage(lang: unknown);
    get updateComplete(): boolean;
    get $$updatesCount(): number | null;
    loadedApps: Map<string, [string | undefined, LoadableAppContext<T, E>]>;
    loadedRoutes: Map<string, string>;
    applicationMapping: Map<string, string>;
    matchCache: MatchCache<T, E>;
    get microfrontendLoader(): LoaderApi;
    get instance(): string;
    protected _extensions: T;
    getApi: () => MicrolcApi<T, E>;
    protected _style: HTMLStyleElement;
    protected _container: HTMLDivElement;
    constructor();
    connectedCallback(): void;
    attributeChangedCallback(name: ObservedAttributes, oldValue: string | null, newValue: unknown): void;
    update(): Promise<boolean>;
    render(): Promise<void>;
    disconnectedCallback(): void;
}
export {};
